package com.tzld.longarticle.recommend.server.mq;

import com.alibaba.fastjson.JSON;
import com.aliyun.openservices.ons.api.Message;
import com.aliyun.openservices.ons.api.SendResult;
import com.aliyun.openservices.ons.api.bean.ProducerBean;
import com.tzld.longarticle.recommend.server.model.param.CallbackParam;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

import java.nio.charset.StandardCharsets;

@Slf4j
@Component
public class MessageCallbackProducer {

    @Value("${pushMessage.callback.topic}")
    private String TOPIC;

    @Value("${pushMessage.callback.tag}")
    private String TAG;

    @Autowired
    private ProducerBean producer;

    public void sendMessage(CallbackParam param) {
        Message message = new Message();
        message.setTopic(TOPIC);
        message.setTag(TAG);
        message.setBody(JSON.toJSONString(param).getBytes(StandardCharsets.UTF_8));
        try {
            SendResult sendResult = producer.send(message);
            log.info("sendResult = {}", sendResult);
        } catch (Exception e) {
            log.error("MessageCallbackProducer send param = {} error", param, e);
            //重试
            retry(message);
        }

    }

    private void retry(Message message) {
        for (int i = 0; i < 3; i++) {
            try {
                SendResult sendResult = producer.send(message);
                log.info("sendResult = {}", sendResult);
                return;
            } catch (Exception e) {
                log.error("MessageCallbackProducer send error {}", i, e);
            }
        }
    }
}
