package com.tzld.longarticle.recommend.server.service.exterior.impl;

import cn.hutool.core.lang.UUID;
import com.tzld.longarticle.recommend.server.common.enums.SecretEnum;
import com.tzld.longarticle.recommend.server.model.param.AccessTokenParam;
import com.tzld.longarticle.recommend.server.model.vo.AccessTokenVo;
import com.tzld.longarticle.recommend.server.service.exterior.AccessTokenService;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;

import java.util.Map;
import java.util.concurrent.TimeUnit;

@Service
public class AccessTokenServiceImpl implements AccessTokenService {

    private static final String ACCESS_TOKEN_LIST = "access_token_list";

    @Autowired
    private RedisTemplate<String, String> redisTemplate;

    @Override
    public AccessTokenVo getAccessToken(AccessTokenParam param) {
        if(param == null || StringUtils.isEmpty(param.getSecret())){
            throw new RuntimeException("参数错误");
        }
        if (!SecretEnum.contains(param.getSecret())) {
            throw new RuntimeException("secret 不存在");
        }
        AccessTokenVo accessTokenVo = new AccessTokenVo();
        String accessToken = redisTemplate.opsForValue().get(param.getSecret());
        if (StringUtils.isNotEmpty(accessToken)) {
            Long expire = redisTemplate.getExpire(param.getSecret(), TimeUnit.SECONDS);
            accessTokenVo.setAccessToken(accessToken);
            accessTokenVo.setExpires(expire);
            if (redisTemplate.opsForHash().size(ACCESS_TOKEN_LIST) > 20) {
                Map<Object, Object> entries = redisTemplate.opsForHash().entries(ACCESS_TOKEN_LIST);
                entries.forEach((k, v) -> {
                    Long timestamp = (Long) v;
                    if (System.currentTimeMillis() / 1000 > timestamp) {
                        redisTemplate.opsForHash().delete(ACCESS_TOKEN_LIST, k);
                    }
                });
            }
        } else {
            String newAccessToken = UUID.randomUUID().toString().replace("-", "");
            redisTemplate.opsForValue().set(param.getSecret(), newAccessToken, 2L, TimeUnit.HOURS);
            long expire = 7200L;
            long timestamp = System.currentTimeMillis() / 1000 + expire + 600;
            redisTemplate.opsForHash().put(ACCESS_TOKEN_LIST, newAccessToken, timestamp);
            accessTokenVo.setAccessToken(newAccessToken);
            accessTokenVo.setExpires(expire);
        }
        return accessTokenVo;
    }

    @Override
    public boolean validateAccessToken(String accessToken) {
        Object o = redisTemplate.opsForHash().get(ACCESS_TOKEN_LIST, accessToken);
        if (o == null) {
            return false;
        }
        Long timestamp = (Long) o;
        if (System.currentTimeMillis() / 1000 > timestamp) {
            redisTemplate.opsForHash().delete(ACCESS_TOKEN_LIST, accessToken);
            return false;
        }
        return true;
    }
}
