package com.tzld.longarticle.recommend.server.service.exterior.impl;

import com.tzld.longarticle.recommend.server.common.enums.cgi.ReplyStrategyServiceEnum;
import com.tzld.longarticle.recommend.server.model.bo.MiniData;
import com.tzld.longarticle.recommend.server.model.cgi.BucketDataParam;
import com.tzld.longarticle.recommend.server.model.cgi.GroupData;
import com.tzld.longarticle.recommend.server.model.cgi.MsgData;
import com.tzld.longarticle.recommend.server.model.cgi.ReplyBucketData;
import com.tzld.longarticle.recommend.server.model.param.PushMessageParam;
import com.tzld.longarticle.recommend.server.model.vo.PushMessageVo;
import com.tzld.longarticle.recommend.server.service.exterior.AccessTokenService;
import com.tzld.longarticle.recommend.server.service.exterior.ThirdPartyService;
import com.tzld.longarticle.recommend.server.service.strategy.reply.ReplyStrategyService;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.ApplicationContext;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import javax.annotation.PostConstruct;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

@Slf4j
@Service
public class ThirdPartyServiceImpl implements ThirdPartyService {

    @Autowired
    private ApplicationContext applicationContext;

    @Autowired
    private AccessTokenService accessTokenService;

    private Map<String, ReplyStrategyService> strategyServiceMap;

    @PostConstruct
    public void init() {
        strategyServiceMap = applicationContext.getBeansOfType(ReplyStrategyService.class);
    }


    @Override
    public List<PushMessageVo> getPushMessage(PushMessageParam param) {
        if (param == null || StringUtils.isEmpty(param.getGhId()) || StringUtils.isEmpty(param.getAccessToken())) {
            throw new RuntimeException("参数错误");
        }
        if (!accessTokenService.validateAccessToken(param.getAccessToken())) {
            throw new RuntimeException("accessToken错误");
        }
        List<PushMessageVo> pushMessageVoList = new ArrayList<>();
        ReplyBucketData replyBucketData = getPushMessageData(param);
        if (replyBucketData == null) {
            log.error("获取推送策略数据失败");
            return pushMessageVoList;
        }
        List<GroupData> groupList = replyBucketData.getGroupList();
        for (GroupData groupData : groupList) {
            if(CollectionUtils.isEmpty(groupData.getMsgDataList())){
                continue;
            }
            PushMessageVo pushMessageVo = new PushMessageVo();
            List<MiniData> components = new ArrayList<>();
            for (MsgData msgData : groupData.getMsgDataList()){
                MiniData miniData = new MiniData();
                BeanUtils.copyProperties(miniData, msgData);
                components.add(miniData);
            }
            pushMessageVo.setGroupIdx(groupData.getGroupIndex());
            pushMessageVo.setComponents(components);
            pushMessageVoList.add(pushMessageVo);
        }
        return pushMessageVoList;
    }

    private ReplyBucketData getPushMessageData(PushMessageParam param) {
        for (Map.Entry<String, ReplyStrategyService> stringReplyStrategyServiceEntry : strategyServiceMap.entrySet()) {
            ReplyStrategyService replyStrategyService = stringReplyStrategyServiceEntry.getValue();
            // 使用策略层
            if (replyStrategyService.support(ReplyStrategyServiceEnum.PUSH_MESSAGE_STRATEGY_V1)) {
                BucketDataParam bucketDataParam = new BucketDataParam();
                bucketDataParam.setGhId(param.getGhId());
                return replyStrategyService.getResult(bucketDataParam);
            }
        }
        // 无执行策略 不会走到这里
        return null;
    }
}
